# This file is @generated
import typing as t
from dataclasses import dataclass

from .. import models
from ..models import (
    ApplicationIn,
    ApplicationOut,
    ApplicationPatch,
    ListResponseApplicationOut,
)
from .common import ApiBase, BaseOptions, serialize_params


@dataclass
class ApplicationListOptions(BaseOptions):
    limit: t.Optional[int] = None
    """Limit the number of returned items"""
    iterator: t.Optional[str] = None
    """The iterator returned from a prior invocation"""
    order: t.Optional[models.Ordering] = None
    """The sorting order of the returned items"""

    def _query_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "limit": self.limit,
                "iterator": self.iterator,
                "order": self.order,
            }
        )


@dataclass
class ApplicationCreateOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


class ApplicationAsync(ApiBase):
    async def list(
        self, options: ApplicationListOptions = ApplicationListOptions()
    ) -> ListResponseApplicationOut:
        """List of all the organization's applications."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/app",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return ListResponseApplicationOut.model_validate(response.json())

    async def create(
        self,
        application_in: ApplicationIn,
        options: ApplicationCreateOptions = ApplicationCreateOptions(),
    ) -> ApplicationOut:
        """Create a new application."""
        response = await self._request_asyncio(
            method="post",
            path="/api/v1/app",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=application_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return ApplicationOut.model_validate(response.json())

    async def get_or_create(
        self,
        application_in: ApplicationIn,
        options: ApplicationCreateOptions = ApplicationCreateOptions(),
    ) -> ApplicationOut:
        response = await self._request_asyncio(
            method="post",
            path="/api/v1/app",
            path_params={},
            query_params={"get_if_exists": "true"},
            header_params=options._header_params(),
            json_body=application_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return ApplicationOut.model_validate(response.json())

    async def get(self, app_id: str) -> ApplicationOut:
        """Get an application."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/app/{app_id}",
            path_params={
                "app_id": app_id,
            },
        )
        return ApplicationOut.model_validate(response.json())

    async def update(
        self, app_id: str, application_in: ApplicationIn
    ) -> ApplicationOut:
        """Update an application."""
        response = await self._request_asyncio(
            method="put",
            path="/api/v1/app/{app_id}",
            path_params={
                "app_id": app_id,
            },
            json_body=application_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return ApplicationOut.model_validate(response.json())

    async def delete(self, app_id: str) -> None:
        """Delete an application."""
        await self._request_asyncio(
            method="delete",
            path="/api/v1/app/{app_id}",
            path_params={
                "app_id": app_id,
            },
        )

    async def patch(
        self, app_id: str, application_patch: ApplicationPatch
    ) -> ApplicationOut:
        """Partially update an application."""
        response = await self._request_asyncio(
            method="patch",
            path="/api/v1/app/{app_id}",
            path_params={
                "app_id": app_id,
            },
            json_body=application_patch.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return ApplicationOut.model_validate(response.json())


class Application(ApiBase):
    def list(
        self, options: ApplicationListOptions = ApplicationListOptions()
    ) -> ListResponseApplicationOut:
        """List of all the organization's applications."""
        response = self._request_sync(
            method="get",
            path="/api/v1/app",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return ListResponseApplicationOut.model_validate(response.json())

    def create(
        self,
        application_in: ApplicationIn,
        options: ApplicationCreateOptions = ApplicationCreateOptions(),
    ) -> ApplicationOut:
        """Create a new application."""
        response = self._request_sync(
            method="post",
            path="/api/v1/app",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=application_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return ApplicationOut.model_validate(response.json())

    def get_or_create(
        self,
        application_in: ApplicationIn,
        options: ApplicationCreateOptions = ApplicationCreateOptions(),
    ) -> ApplicationOut:
        response = self._request_sync(
            method="post",
            path="/api/v1/app",
            path_params={},
            query_params={"get_if_exists": "true"},
            header_params=options._header_params(),
            json_body=application_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return ApplicationOut.model_validate(response.json())

    def get(self, app_id: str) -> ApplicationOut:
        """Get an application."""
        response = self._request_sync(
            method="get",
            path="/api/v1/app/{app_id}",
            path_params={
                "app_id": app_id,
            },
        )
        return ApplicationOut.model_validate(response.json())

    def update(self, app_id: str, application_in: ApplicationIn) -> ApplicationOut:
        """Update an application."""
        response = self._request_sync(
            method="put",
            path="/api/v1/app/{app_id}",
            path_params={
                "app_id": app_id,
            },
            json_body=application_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return ApplicationOut.model_validate(response.json())

    def delete(self, app_id: str) -> None:
        """Delete an application."""
        self._request_sync(
            method="delete",
            path="/api/v1/app/{app_id}",
            path_params={
                "app_id": app_id,
            },
        )

    def patch(self, app_id: str, application_patch: ApplicationPatch) -> ApplicationOut:
        """Partially update an application."""
        response = self._request_sync(
            method="patch",
            path="/api/v1/app/{app_id}",
            path_params={
                "app_id": app_id,
            },
            json_body=application_patch.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return ApplicationOut.model_validate(response.json())



# this file is @generated
from enum import Enum


class BackgroundTaskStatus(str, Enum):
    RUNNING = "running"
    FINISHED = "finished"
    FAILED = "failed"

    def __str__(self) -> str:
        return str(self.value)



# this file is @generated
from enum import Enum


class Ordering(str, Enum):
    """Defines the ordering in a listing of results."""

    ASCENDING = "ascending"
    DESCENDING = "descending"

    def __str__(self) -> str:
        return str(self.value)



# this file is @generated
import typing as t

from .common import BaseModel
from .connector_in import ConnectorIn
from .event_type_in import EventTypeIn


class EnvironmentIn(BaseModel):
    connectors: t.Optional[t.List[ConnectorIn]] = None

    event_types: t.Optional[t.List[EventTypeIn]] = None

    settings: t.Optional[t.Dict[str, t.Any]] = None



# this file is @generated
import typing as t

from .common import BaseModel


class ApplicationTokenExpireIn(BaseModel):
    expiry: t.Optional[int] = None
    """How many seconds until the old key is expired."""



# This file is @generated
import typing as t
from dataclasses import dataclass
from datetime import datetime

from .. import models
from ..models import (
    EndpointHeadersIn,
    EndpointHeadersOut,
    EndpointHeadersPatchIn,
    EndpointIn,
    EndpointOut,
    EndpointPatch,
    EndpointSecretOut,
    EndpointSecretRotateIn,
    EndpointStats,
    EndpointTransformationIn,
    EndpointTransformationOut,
    EndpointUpdate,
    EventExampleIn,
    ListResponseEndpointOut,
    MessageOut,
    RecoverIn,
    RecoverOut,
    ReplayIn,
    ReplayOut,
)
from .common import ApiBase, BaseOptions, serialize_params


@dataclass
class EndpointListOptions(BaseOptions):
    limit: t.Optional[int] = None
    """Limit the number of returned items"""
    iterator: t.Optional[str] = None
    """The iterator returned from a prior invocation"""
    order: t.Optional[models.Ordering] = None
    """The sorting order of the returned items"""

    def _query_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "limit": self.limit,
                "iterator": self.iterator,
                "order": self.order,
            }
        )


@dataclass
class EndpointCreateOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


@dataclass
class EndpointRecoverOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


@dataclass
class EndpointReplayMissingOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


@dataclass
class EndpointRotateSecretOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


@dataclass
class EndpointSendExampleOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


@dataclass
class EndpointGetStatsOptions(BaseOptions):
    since: t.Optional[datetime] = None
    """Filter the range to data starting from this date."""
    until: t.Optional[datetime] = None
    """Filter the range to data ending by this date."""

    def _query_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "since": self.since,
                "until": self.until,
            }
        )


class EndpointAsync(ApiBase):
    async def list(
        self, app_id: str, options: EndpointListOptions = EndpointListOptions()
    ) -> ListResponseEndpointOut:
        """List the application's endpoints."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/app/{app_id}/endpoint",
            path_params={
                "app_id": app_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return ListResponseEndpointOut.model_validate(response.json())

    async def create(
        self,
        app_id: str,
        endpoint_in: EndpointIn,
        options: EndpointCreateOptions = EndpointCreateOptions(),
    ) -> EndpointOut:
        """Create a new endpoint for the application.

        When `secret` is `null` the secret is automatically generated (recommended)."""
        response = await self._request_asyncio(
            method="post",
            path="/api/v1/app/{app_id}/endpoint",
            path_params={
                "app_id": app_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=endpoint_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return EndpointOut.model_validate(response.json())

    async def get(self, app_id: str, endpoint_id: str) -> EndpointOut:
        """Get an endpoint."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
        )
        return EndpointOut.model_validate(response.json())

    async def update(
        self, app_id: str, endpoint_id: str, endpoint_update: EndpointUpdate
    ) -> EndpointOut:
        """Update an endpoint."""
        response = await self._request_asyncio(
            method="put",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
            json_body=endpoint_update.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return EndpointOut.model_validate(response.json())

    async def delete(self, app_id: str, endpoint_id: str) -> None:
        """Delete an endpoint."""
        await self._request_asyncio(
            method="delete",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
        )

    async def patch(
        self, app_id: str, endpoint_id: str, endpoint_patch: EndpointPatch
    ) -> EndpointOut:
        """Partially update an endpoint."""
        response = await self._request_asyncio(
            method="patch",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
            json_body=endpoint_patch.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return EndpointOut.model_validate(response.json())

    async def get_headers(self, app_id: str, endpoint_id: str) -> EndpointHeadersOut:
        """Get the additional headers to be sent with the webhook."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/headers",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
        )
        return EndpointHeadersOut.model_validate(response.json())

    async def update_headers(
        self, app_id: str, endpoint_id: str, endpoint_headers_in: EndpointHeadersIn
    ) -> None:
        """Set the additional headers to be sent with the webhook."""
        await self._request_asyncio(
            method="put",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/headers",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
            json_body=endpoint_headers_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )

    async def patch_headers(
        self,
        app_id: str,
        endpoint_id: str,
        endpoint_headers_patch_in: EndpointHeadersPatchIn,
    ) -> None:
        """Partially set the additional headers to be sent with the webhook."""
        await self._request_asyncio(
            method="patch",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/headers",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
            json_body=endpoint_headers_patch_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )

    async def recover(
        self,
        app_id: str,
        endpoint_id: str,
        recover_in: RecoverIn,
        options: EndpointRecoverOptions = EndpointRecoverOptions(),
    ) -> RecoverOut:
        """Resend all failed messages since a given time.

        Messages that were sent successfully, even if failed initially, are not resent."""
        response = await self._request_asyncio(
            method="post",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/recover",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=recover_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return RecoverOut.model_validate(response.json())

    async def replay_missing(
        self,
        app_id: str,
        endpoint_id: str,
        replay_in: ReplayIn,
        options: EndpointReplayMissingOptions = EndpointReplayMissingOptions(),
    ) -> ReplayOut:
        """Replays messages to the endpoint.

        Only messages that were created after `since` will be sent.
        Messages that were previously sent to the endpoint are not resent."""
        response = await self._request_asyncio(
            method="post",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/replay-missing",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=replay_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return ReplayOut.model_validate(response.json())

    async def get_secret(self, app_id: str, endpoint_id: str) -> EndpointSecretOut:
        """Get the endpoint's signing secret.

        This is used to verify the authenticity of the webhook.
        For more information please refer to [the consuming webhooks docs](https://docs.svix.com/consuming-webhooks/)."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/secret",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
        )
        return EndpointSecretOut.model_validate(response.json())

    async def rotate_secret(
        self,
        app_id: str,
        endpoint_id: str,
        endpoint_secret_rotate_in: EndpointSecretRotateIn,
        options: EndpointRotateSecretOptions = EndpointRotateSecretOptions(),
    ) -> None:
        """Rotates the endpoint's signing secret.

        The previous secret will remain valid for the next 24 hours."""
        await self._request_asyncio(
            method="post",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/secret/rotate",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=endpoint_secret_rotate_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )

    async def send_example(
        self,
        app_id: str,
        endpoint_id: str,
        event_example_in: EventExampleIn,
        options: EndpointSendExampleOptions = EndpointSendExampleOptions(),
    ) -> MessageOut:
        """Send an example message for an event."""
        response = await self._request_asyncio(
            method="post",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/send-example",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=event_example_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return MessageOut.model_validate(response.json())

    async def get_stats(
        self,
        app_id: str,
        endpoint_id: str,
        options: EndpointGetStatsOptions = EndpointGetStatsOptions(),
    ) -> EndpointStats:
        """Get basic statistics for the endpoint."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/stats",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return EndpointStats.model_validate(response.json())

    async def transformation_get(
        self, app_id: str, endpoint_id: str
    ) -> EndpointTransformationOut:
        """Get the transformation code associated with this endpoint."""
        response = await self._request_asyncio(
            method="get",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/transformation",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
        )
        return EndpointTransformationOut.model_validate(response.json())

    async def transformation_partial_update(
        self,
        app_id: str,
        endpoint_id: str,
        endpoint_transformation_in: EndpointTransformationIn,
    ) -> None:
        """Set or unset the transformation code associated with this endpoint."""
        await self._request_asyncio(
            method="patch",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/transformation",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
            json_body=endpoint_transformation_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )


class Endpoint(ApiBase):
    def list(
        self, app_id: str, options: EndpointListOptions = EndpointListOptions()
    ) -> ListResponseEndpointOut:
        """List the application's endpoints."""
        response = self._request_sync(
            method="get",
            path="/api/v1/app/{app_id}/endpoint",
            path_params={
                "app_id": app_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return ListResponseEndpointOut.model_validate(response.json())

    def create(
        self,
        app_id: str,
        endpoint_in: EndpointIn,
        options: EndpointCreateOptions = EndpointCreateOptions(),
    ) -> EndpointOut:
        """Create a new endpoint for the application.

        When `secret` is `null` the secret is automatically generated (recommended)."""
        response = self._request_sync(
            method="post",
            path="/api/v1/app/{app_id}/endpoint",
            path_params={
                "app_id": app_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=endpoint_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return EndpointOut.model_validate(response.json())

    def get(self, app_id: str, endpoint_id: str) -> EndpointOut:
        """Get an endpoint."""
        response = self._request_sync(
            method="get",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
        )
        return EndpointOut.model_validate(response.json())

    def update(
        self, app_id: str, endpoint_id: str, endpoint_update: EndpointUpdate
    ) -> EndpointOut:
        """Update an endpoint."""
        response = self._request_sync(
            method="put",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
            json_body=endpoint_update.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return EndpointOut.model_validate(response.json())

    def delete(self, app_id: str, endpoint_id: str) -> None:
        """Delete an endpoint."""
        self._request_sync(
            method="delete",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
        )

    def patch(
        self, app_id: str, endpoint_id: str, endpoint_patch: EndpointPatch
    ) -> EndpointOut:
        """Partially update an endpoint."""
        response = self._request_sync(
            method="patch",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
            json_body=endpoint_patch.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return EndpointOut.model_validate(response.json())

    def get_headers(self, app_id: str, endpoint_id: str) -> EndpointHeadersOut:
        """Get the additional headers to be sent with the webhook."""
        response = self._request_sync(
            method="get",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/headers",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
        )
        return EndpointHeadersOut.model_validate(response.json())

    def update_headers(
        self, app_id: str, endpoint_id: str, endpoint_headers_in: EndpointHeadersIn
    ) -> None:
        """Set the additional headers to be sent with the webhook."""
        self._request_sync(
            method="put",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/headers",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
            json_body=endpoint_headers_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )

    def patch_headers(
        self,
        app_id: str,
        endpoint_id: str,
        endpoint_headers_patch_in: EndpointHeadersPatchIn,
    ) -> None:
        """Partially set the additional headers to be sent with the webhook."""
        self._request_sync(
            method="patch",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/headers",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
            json_body=endpoint_headers_patch_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )

    def recover(
        self,
        app_id: str,
        endpoint_id: str,
        recover_in: RecoverIn,
        options: EndpointRecoverOptions = EndpointRecoverOptions(),
    ) -> RecoverOut:
        """Resend all failed messages since a given time.

        Messages that were sent successfully, even if failed initially, are not resent."""
        response = self._request_sync(
            method="post",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/recover",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=recover_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return RecoverOut.model_validate(response.json())

    def replay_missing(
        self,
        app_id: str,
        endpoint_id: str,
        replay_in: ReplayIn,
        options: EndpointReplayMissingOptions = EndpointReplayMissingOptions(),
    ) -> ReplayOut:
        """Replays messages to the endpoint.

        Only messages that were created after `since` will be sent.
        Messages that were previously sent to the endpoint are not resent."""
        response = self._request_sync(
            method="post",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/replay-missing",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=replay_in.model_dump_json(exclude_unset=True, by_alias=True),
        )
        return ReplayOut.model_validate(response.json())

    def get_secret(self, app_id: str, endpoint_id: str) -> EndpointSecretOut:
        """Get the endpoint's signing secret.

        This is used to verify the authenticity of the webhook.
        For more information please refer to [the consuming webhooks docs](https://docs.svix.com/consuming-webhooks/)."""
        response = self._request_sync(
            method="get",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/secret",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
        )
        return EndpointSecretOut.model_validate(response.json())

    def rotate_secret(
        self,
        app_id: str,
        endpoint_id: str,
        endpoint_secret_rotate_in: EndpointSecretRotateIn,
        options: EndpointRotateSecretOptions = EndpointRotateSecretOptions(),
    ) -> None:
        """Rotates the endpoint's signing secret.

        The previous secret will remain valid for the next 24 hours."""
        self._request_sync(
            method="post",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/secret/rotate",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=endpoint_secret_rotate_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )

    def send_example(
        self,
        app_id: str,
        endpoint_id: str,
        event_example_in: EventExampleIn,
        options: EndpointSendExampleOptions = EndpointSendExampleOptions(),
    ) -> MessageOut:
        """Send an example message for an event."""
        response = self._request_sync(
            method="post",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/send-example",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=event_example_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return MessageOut.model_validate(response.json())

    def get_stats(
        self,
        app_id: str,
        endpoint_id: str,
        options: EndpointGetStatsOptions = EndpointGetStatsOptions(),
    ) -> EndpointStats:
        """Get basic statistics for the endpoint."""
        response = self._request_sync(
            method="get",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/stats",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
            query_params=options._query_params(),
            header_params=options._header_params(),
        )
        return EndpointStats.model_validate(response.json())

    def transformation_get(
        self, app_id: str, endpoint_id: str
    ) -> EndpointTransformationOut:
        """Get the transformation code associated with this endpoint."""
        response = self._request_sync(
            method="get",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/transformation",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
        )
        return EndpointTransformationOut.model_validate(response.json())

    def transformation_partial_update(
        self,
        app_id: str,
        endpoint_id: str,
        endpoint_transformation_in: EndpointTransformationIn,
    ) -> None:
        """Set or unset the transformation code associated with this endpoint."""
        self._request_sync(
            method="patch",
            path="/api/v1/app/{app_id}/endpoint/{endpoint_id}/transformation",
            path_params={
                "app_id": app_id,
                "endpoint_id": endpoint_id,
            },
            json_body=endpoint_transformation_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )



import svix
from svix.api import MessageListOptions
import httpretty

@httpretty.activate(verbose=True, allow_net_connect=False)
def test_octothorpe_in_query_param():
    svx = svix.Svix("token",svix.SvixOptions(server_url="http://test.example"))
    httpretty.register_uri(
        httpretty.GET,
        "http://test.example/api/v1/app/app_id/msg?tag=test%23test",
        body='{"data":[],"iterator":null,"prevIterator":null,"done":true}'
    )
    svx.message.list("app_id",MessageListOptions(tag="test#test"))



# this file is @generated
import typing as t

from .common import BaseModel


class EndpointSecretRotateIn(BaseModel):
    key: t.Optional[str] = None
    """The endpoint's verification secret.

    Format: `base64` encoded random bytes optionally prefixed with `whsec_`.
    It is recommended to not set this and let the server generate the secret."""



# This file is @generated
import typing as t
from dataclasses import dataclass

from ..models import AggregateEventTypesOut, AppUsageStatsIn, AppUsageStatsOut
from .common import ApiBase, BaseOptions, serialize_params


@dataclass
class StatisticsAggregateAppStatsOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None

    def _header_params(self) -> t.Dict[str, str]:
        return serialize_params(
            {
                "idempotency-key": self.idempotency_key,
            }
        )


class StatisticsAsync(ApiBase):
    async def aggregate_app_stats(
        self,
        app_usage_stats_in: AppUsageStatsIn,
        options: StatisticsAggregateAppStatsOptions = StatisticsAggregateAppStatsOptions(),
    ) -> AppUsageStatsOut:
        """Creates a background task to calculate the message destinations for all applications in the environment.

        Note that this endpoint is asynchronous. You will need to poll the `Get Background Task` endpoint to
        retrieve the results of the operation."""
        response = await self._request_asyncio(
            method="post",
            path="/api/v1/stats/usage/app",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=app_usage_stats_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return AppUsageStatsOut.model_validate(response.json())

    async def aggregate_event_types(self) -> AggregateEventTypesOut:
        """Creates a background task to calculate the listed event types for all apps in the organization.

        Note that this endpoint is asynchronous. You will need to poll the `Get Background Task` endpoint to
        retrieve the results of the operation."""
        response = await self._request_asyncio(
            method="put", path="/api/v1/stats/usage/event-types", path_params={}
        )
        return AggregateEventTypesOut.model_validate(response.json())


class Statistics(ApiBase):
    def aggregate_app_stats(
        self,
        app_usage_stats_in: AppUsageStatsIn,
        options: StatisticsAggregateAppStatsOptions = StatisticsAggregateAppStatsOptions(),
    ) -> AppUsageStatsOut:
        """Creates a background task to calculate the message destinations for all applications in the environment.

        Note that this endpoint is asynchronous. You will need to poll the `Get Background Task` endpoint to
        retrieve the results of the operation."""
        response = self._request_sync(
            method="post",
            path="/api/v1/stats/usage/app",
            path_params={},
            query_params=options._query_params(),
            header_params=options._header_params(),
            json_body=app_usage_stats_in.model_dump_json(
                exclude_unset=True, by_alias=True
            ),
        )
        return AppUsageStatsOut.model_validate(response.json())

    def aggregate_event_types(self) -> AggregateEventTypesOut:
        """Creates a background task to calculate the listed event types for all apps in the organization.

        Note that this endpoint is asynchronous. You will need to poll the `Get Background Task` endpoint to
        retrieve the results of the operation."""
        response = self._request_sync(
            method="put", path="/api/v1/stats/usage/event-types", path_params={}
        )
        return AggregateEventTypesOut.model_validate(response.json())



import typing as t
from dataclasses import dataclass, field

# add __init__.py from models directory
from ..models import (
    AggregateEventTypesOut,
    ApplicationIn,
    ApplicationOut,
    ApplicationPatch,
    ApplicationTokenExpireIn,
    AppPortalAccessIn,
    AppPortalAccessOut,
    AppUsageStatsIn,
    AppUsageStatsOut,
    BackgroundTaskData,
    BackgroundTaskOut,
    BackgroundTaskStatus,
    BackgroundTaskType,
    ConnectorIn,
    ConnectorKind,
    DashboardAccessOut,
    EndpointHeadersIn,
    EndpointHeadersOut,
    EndpointHeadersPatchIn,
    EndpointIn,
    EndpointMessageOut,
    EndpointOut,
    EndpointPatch,
    EndpointSecretOut,
    EndpointSecretRotateIn,
    EndpointStats,
    EndpointTransformationIn,
    EndpointTransformationOut,
    EndpointUpdate,
    EnvironmentIn,
    EnvironmentOut,
    EventExampleIn,
    EventTypeFromOpenApi,
    EventTypeImportOpenApiIn,
    EventTypeImportOpenApiOut,
    EventTypeImportOpenApiOutData,
    EventTypeIn,
    EventTypeOut,
    EventTypePatch,
    EventTypeUpdate,
    IntegrationIn,
    IntegrationKeyOut,
    IntegrationOut,
    IntegrationUpdate,
    ListResponseApplicationOut,
    ListResponseBackgroundTaskOut,
    ListResponseEndpointMessageOut,
    ListResponseEndpointOut,
    ListResponseEventTypeOut,
    ListResponseIntegrationOut,
    ListResponseMessageAttemptOut,
    ListResponseMessageEndpointOut,
    ListResponseMessageOut,
    ListResponseOperationalWebhookEndpointOut,
    MessageAttemptOut,
    MessageAttemptTriggerType,
    MessageEndpointOut,
    MessageIn,
    MessageOut,
    MessageStatus,
    OperationalWebhookEndpointHeadersIn,
    OperationalWebhookEndpointHeadersOut,
    OperationalWebhookEndpointIn,
    OperationalWebhookEndpointOut,
    OperationalWebhookEndpointSecretIn,
    OperationalWebhookEndpointSecretOut,
    OperationalWebhookEndpointUpdate,
    Ordering,
    RecoverIn,
    RecoverOut,
    ReplayIn,
    ReplayOut,
    StatusCodeClass,
    TemplateOut,
)
from .application import (
    Application,
    ApplicationAsync,
    ApplicationCreateOptions,
    ApplicationListOptions,
)
from .authentication import (
    Authentication,
    AuthenticationAppPortalAccessOptions,
    AuthenticationAsync,
    AuthenticationDashboardAccessOptions,
    AuthenticationExpireAllOptions,
    AuthenticationLogoutOptions,
)
from .background_task import (
    BackgroundTask,
    BackgroundTaskAsync,
    BackgroundTaskListOptions,
)
from .client import AuthenticatedClient
from .endpoint import (
    Endpoint,
    EndpointAsync,
    EndpointCreateOptions,
    EndpointGetStatsOptions,
    EndpointListOptions,
    EndpointRecoverOptions,
    EndpointReplayMissingOptions,
    EndpointRotateSecretOptions,
    EndpointSendExampleOptions,
)
from .event_type import (
    EventType,
    EventTypeAsync,
    EventTypeCreateOptions,
    EventTypeDeleteOptions,
    EventTypeImportOpenapiOptions,
    EventTypeListOptions,
)
from .integration import (
    Integration,
    IntegrationAsync,
    IntegrationCreateOptions,
    IntegrationListOptions,
    IntegrationRotateKeyOptions,
)
from .message import (
    Message,
    MessageAsync,
    MessageCreateOptions,
    MessageExpungeAllContentsOptions,
    MessageGetOptions,
    MessageListOptions,
)
from .message_attempt import (
    MessageAttempt,
    MessageAttemptAsync,
    MessageAttemptListAttemptedDestinationsOptions,
    MessageAttemptListAttemptedMessagesOptions,
    MessageAttemptListByEndpointOptions,
    MessageAttemptListByMsgOptions,
    MessageAttemptResendOptions,
)
from .operational_webhook_endpoint import (
    OperationalWebhookEndpoint,
    OperationalWebhookEndpointAsync,
    OperationalWebhookEndpointCreateOptions,
    OperationalWebhookEndpointListOptions,
    OperationalWebhookEndpointRotateSecretOptions,
)
from .statistics import Statistics, StatisticsAggregateAppStatsOptions, StatisticsAsync

DEFAULT_SERVER_URL = "https://api.svix.com"


@dataclass
class SvixOptions:
    debug: bool = False
    server_url: t.Optional[str] = None
    """
    The retry schedule, as seconds to wait after each failed request.

    The first entry is the time in seconds to wait between the first request
    failing and the first retry, and so on.
    Up to five retries are supported, passing a retry schedule with more than
    five entries will raise a `ValueError`.

    Defaults to [0.05, 0.1, 0.2]
    """
    retry_schedule: t.List[float] = field(default_factory=lambda: [0.05, 0.1, 0.2])

    """
    The maximum amount of time in seconds a request can take.

    Request methods will raise httpx.TimeoutException if this is exceeded.
    """
    timeout: float = 15.0


class ClientBase:
    _client: AuthenticatedClient

    def __init__(self, auth_token: str, options: SvixOptions = SvixOptions()) -> None:
        from .. import __version__

        if len(options.retry_schedule) > 5:
            raise ValueError("number of retries must not exceed 5")

        regional_url = None
        region = auth_token.split(".")[-1]
        if region == "us":
            regional_url = "https://api.us.svix.com"
        elif region == "eu":
            regional_url = "https://api.eu.svix.com"
        elif region == "in":
            regional_url = "https://api.in.svix.com"

        host = options.server_url or regional_url or DEFAULT_SERVER_URL
        client = AuthenticatedClient(
            base_url=host,
            token=auth_token,
            headers={"user-agent": f"svix-libs/{__version__}/python"},
            verify_ssl=True,
            retry_schedule=options.retry_schedule,
            timeout=options.timeout,
            follow_redirects=False,
            raise_on_unexpected_status=True,
        )
        self._client = client


class SvixAsync(ClientBase):
    @property
    def authentication(self) -> AuthenticationAsync:
        return AuthenticationAsync(self._client)

    @property
    def application(self) -> ApplicationAsync:
        return ApplicationAsync(self._client)

    @property
    def endpoint(self) -> EndpointAsync:
        return EndpointAsync(self._client)

    @property
    def event_type(self) -> EventTypeAsync:
        return EventTypeAsync(self._client)

    @property
    def integration(self) -> IntegrationAsync:
        return IntegrationAsync(self._client)

    @property
    def message(self) -> MessageAsync:
        return MessageAsync(self._client)

    @property
    def message_attempt(self) -> MessageAttemptAsync:
        return MessageAttemptAsync(self._client)

    @property
    def statistics(self) -> StatisticsAsync:
        return StatisticsAsync(self._client)

    @property
    def operational_webhook_endpoint(self) -> OperationalWebhookEndpointAsync:
        return OperationalWebhookEndpointAsync(self._client)


class Svix(ClientBase):
    @property
    def authentication(self) -> Authentication:
        return Authentication(self._client)

    @property
    def application(self) -> Application:
        return Application(self._client)

    @property
    def endpoint(self) -> Endpoint:
        return Endpoint(self._client)

    @property
    def event_type(self) -> EventType:
        return EventType(self._client)

    @property
    def integration(self) -> Integration:
        return Integration(self._client)

    @property
    def message(self) -> Message:
        return Message(self._client)

    @property
    def message_attempt(self) -> MessageAttempt:
        return MessageAttempt(self._client)

    @property
    def statistics(self) -> Statistics:
        return Statistics(self._client)

    @property
    def operational_webhook_endpoint(self) -> OperationalWebhookEndpoint:
        return OperationalWebhookEndpoint(self._client)


__all__ = [
    "AuthenticatedClient",
    "Application",
    "ApplicationAsync",
    "ApplicationListOptions",
    "ApplicationCreateOptions",
    "Authentication",
    "AuthenticationAsync",
    "AuthenticationAppPortalAccessOptions",
    "AuthenticationExpireAllOptions",
    "AuthenticationDashboardAccessOptions",
    "AuthenticationLogoutOptions",
    "BackgroundTask",
    "BackgroundTaskAsync",
    "BackgroundTaskListOptions",
    "Endpoint",
    "EndpointAsync",
    "EndpointListOptions",
    "EndpointCreateOptions",
    "EndpointRecoverOptions",
    "EndpointReplayMissingOptions",
    "EndpointRotateSecretOptions",
    "EndpointSendExampleOptions",
    "EndpointGetStatsOptions",
    "EventType",
    "EventTypeAsync",
    "EventTypeListOptions",
    "EventTypeCreateOptions",
    "EventTypeImportOpenapiOptions",
    "EventTypeDeleteOptions",
    "Integration",
    "IntegrationAsync",
    "IntegrationListOptions",
    "IntegrationCreateOptions",
    "IntegrationRotateKeyOptions",
    "Message",
    "MessageAsync",
    "MessageListOptions",
    "MessageCreateOptions",
    "MessageExpungeAllContentsOptions",
    "MessageGetOptions",
    "MessageAttempt",
    "MessageAttemptAsync",
    "MessageAttemptListByEndpointOptions",
    "MessageAttemptListByMsgOptions",
    "MessageAttemptListAttemptedMessagesOptions",
    "MessageAttemptListAttemptedDestinationsOptions",
    "MessageAttemptResendOptions",
    "OperationalWebhookEndpoint",
    "OperationalWebhookEndpointAsync",
    "OperationalWebhookEndpointListOptions",
    "OperationalWebhookEndpointCreateOptions",
    "OperationalWebhookEndpointRotateSecretOptions",
    "Statistics",
    "StatisticsAsync",
    "StatisticsAggregateAppStatsOptions",
    # These are edited in by hand
    "AggregateEventTypesOut",
    "AppPortalAccessIn",
    "AppPortalAccessOut",
    "AppUsageStatsIn",
    "AppUsageStatsOut",
    "ApplicationIn",
    "ApplicationOut",
    "ApplicationPatch",
    "ApplicationTokenExpireIn",
    "BackgroundTaskData",
    "BackgroundTaskOut",
    "BackgroundTaskStatus",
    "BackgroundTaskType",
    "ConnectorIn",
    "ConnectorKind",
    "DashboardAccessOut",
    "EndpointHeadersIn",
    "EndpointHeadersOut",
    "EndpointHeadersPatchIn",
    "EndpointIn",
    "EndpointMessageOut",
    "EndpointOut",
    "EndpointPatch",
    "EndpointSecretOut",
    "EndpointSecretRotateIn",
    "EndpointStats",
    "EndpointTransformationIn",
    "EndpointTransformationOut",
    "EndpointUpdate",
    "EnvironmentIn",
    "EnvironmentOut",
    "EventExampleIn",
    "EventTypeFromOpenApi",
    "EventTypeImportOpenApiIn",
    "EventTypeImportOpenApiOut",
    "EventTypeImportOpenApiOutData",
    "EventTypeIn",
    "EventTypeOut",
    "EventTypePatch",
    "EventTypeUpdate",
    "IntegrationIn",
    "IntegrationKeyOut",
    "IntegrationOut",
    "IntegrationUpdate",
    "ListResponseApplicationOut",
    "ListResponseBackgroundTaskOut",
    "ListResponseEndpointMessageOut",
    "ListResponseEndpointOut",
    "ListResponseEventTypeOut",
    "ListResponseIntegrationOut",
    "ListResponseMessageAttemptOut",
    "ListResponseMessageEndpointOut",
    "ListResponseMessageOut",
    "ListResponseOperationalWebhookEndpointOut",
    "MessageAttemptOut",
    "MessageAttemptTriggerType",
    "MessageEndpointOut",
    "MessageIn",
    "MessageOut",
    "MessageStatus",
    "OperationalWebhookEndpointHeadersIn",
    "OperationalWebhookEndpointHeadersOut",
    "OperationalWebhookEndpointIn",
    "OperationalWebhookEndpointOut",
    "OperationalWebhookEndpointSecretIn",
    "OperationalWebhookEndpointSecretOut",
    "OperationalWebhookEndpointUpdate",
    "Ordering",
    "RecoverIn",
    "RecoverOut",
    "ReplayIn",
    "ReplayOut",
    "StatusCodeClass",
    "TemplateOut",
]



# this file is @generated
import typing as t
from datetime import datetime

from .common import BaseModel


class ApplicationOut(BaseModel):
    created_at: datetime

    id: str
    """The Application's ID."""

    metadata: t.Dict[str, str]

    name: str

    rate_limit: t.Optional[int] = None

    uid: t.Optional[str] = None
    """The Application's UID."""

    updated_at: datetime



from .api.errors.http_error import HttpError
from .api.errors.http_validation_error import HTTPValidationError
from .webhooks import WebhookVerificationError


__all__ = ["HttpError", "WebhookVerificationError", "HTTPValidationError"]



# this file is @generated

from .background_task_status import BackgroundTaskStatus
from .background_task_type import BackgroundTaskType
from .common import BaseModel


class ReplayOut(BaseModel):
    id: str
    """The QueueBackgroundTask's ID."""

    status: BackgroundTaskStatus

    task: BackgroundTaskType



# this file is @generated
import typing as t
from datetime import datetime

from .common import BaseModel


class IntegrationOut(BaseModel):
    created_at: datetime

    feature_flags: t.Optional[t.List[str]] = None
    """The set of feature flags the integration has access to."""

    id: str
    """The Integration's ID."""

    name: str

    updated_at: datetime



import os

import pytest

from svix import SvixOptions
from svix.api import (
    ApplicationIn,
    EndpointIn,
    EventTypeIn,
    Svix,
)
from svix.exceptions import HttpError
from svix.models import EndpointPatch

TOKEN = os.getenv("SVIX_TOKEN")
SERVER_URL = os.getenv("SVIX_SERVER_URL")

CLIENT = (
    Svix(TOKEN, SvixOptions(server_url=SERVER_URL))
    if TOKEN is not None and SERVER_URL is not None
    else None
)


@pytest.mark.skipif(
    CLIENT is None,
    reason="SVIX_TOKEN and SVIX_SERVER_URL are required to run this test",
)
def test_endpoint_crud() -> None:
    app = CLIENT.application.create(ApplicationIn(name="app"))
    try:
        CLIENT.event_type.create(
            EventTypeIn(name="event.started", description="Something started")
        )
    except HttpError as e:
        assert e.status_code == 409, "conflicts are expected but other statuses are not"
    try:
        CLIENT.event_type.create(
            EventTypeIn(name="event.ended", description="Something ended")
        )
    except HttpError as e:
        assert e.status_code == 409, "conflicts are expected but other statuses are not"

    ep = CLIENT.endpoint.create(
        app.id, EndpointIn(url="https://example.svix.com/", channels=["ch0", "ch1"])
    )
    assert {s for s in ep.channels} == {"ch0", "ch1"}
    ep_patched = CLIENT.endpoint.patch(
        app.id, ep.id, EndpointPatch(filter_types=["event.started", "event.ended"])
    )
    assert {s for s in ep_patched.channels} == {"ch0", "ch1"}
    assert {s for s in ep_patched.filter_types} == {"event.started", "event.ended"}

    # Should succeed without error if the deserialization handles empty response bodies correctly
    CLIENT.endpoint.delete(app.id, ep.id)



# this file is @generated
import typing as t

from .common import BaseModel
from .connector_kind import ConnectorKind


class ConnectorIn(BaseModel):
    description: t.Optional[str] = None

    feature_flag: t.Optional[str] = None

    filter_types: t.Optional[t.List[str]] = None

    instructions: t.Optional[str] = None

    instructions_link: t.Optional[str] = None

    kind: t.Optional[ConnectorKind] = None

    logo: str

    name: str

    transformation: str



# this file is @generated
import typing as t

from .common import BaseModel


class EventTypeImportOpenApiIn(BaseModel):
    """Import a list of event types from webhooks defined in an OpenAPI spec.

    The OpenAPI spec can be specified as either `spec` given the spec as a JSON object, or as `specRaw` (a `string`) which will be parsed as YAML or JSON by the server. Sending neither or both is invalid, resulting in a `400` **Bad Request**."""

    dry_run: t.Optional[bool] = None
    """If `true`, return the event types that would be modified without actually modifying them."""

    replace_all: t.Optional[bool] = None
    """If `true`, all existing event types that are not in the spec will be archived."""

    spec: t.Optional[t.Dict[str, t.Any]] = None
    """A pre-parsed JSON spec."""

    spec_raw: t.Optional[str] = None
    """A string, parsed by the server as YAML or JSON."""



import asyncio
import random
import time
import typing as t
from dataclasses import asdict, dataclass
from datetime import datetime, timezone

import httpx

from .client import AuthenticatedClient
from .errors.http_error import HttpError
from .errors.http_validation_error import HTTPValidationError


def ensure_tz(x: t.Optional[datetime]) -> t.Optional[datetime]:
    if x is None:
        return None

    if x.tzinfo is None:
        return x.replace(tzinfo=timezone.utc)
    return x


def sanitize_field(v: t.Any) -> t.Any:
    if isinstance(v, datetime):
        return ensure_tz(v)

    return v


def _serialize_single_param(val: t.Any) -> str:
    if isinstance(val, datetime):
        if val.tzinfo is None:
            val.replace(tzinfo=timezone.utc)
        return val.isoformat()
    elif isinstance(val, bool):
        return "true" if val else "false"
    elif isinstance(val, set) or isinstance(val, list):
        return ",".join(val)
    else:
        return str(val)


def serialize_params(d: t.Dict[str, t.Optional[t.Any]]) -> t.Dict[str, str]:
    return {k: _serialize_single_param(v) for k, v in d.items() if v is not None}


@dataclass
class BaseOptions:
    def to_dict(self) -> t.Dict[str, t.Any]:
        return {k: sanitize_field(v) for k, v in asdict(self).items() if v is not None}

    def _query_params(self) -> t.Dict[str, str]:
        return {}

    def _header_params(self) -> t.Dict[str, str]:
        return {}


@dataclass
class ListOptions(BaseOptions):
    iterator: t.Optional[str] = None
    limit: t.Optional[int] = None


@dataclass
class PostOptions(BaseOptions):
    idempotency_key: t.Optional[str] = None


class ApiBase:
    _client: AuthenticatedClient
    _httpx_client: httpx.Client
    _httpx_async_client: httpx.AsyncClient

    def __init__(self, client: AuthenticatedClient) -> None:
        self._client = client
        self._httpx_client = httpx.Client(
            verify=client.verify_ssl, cookies=self._client.get_cookies()
        )
        self._httpx_async_client = httpx.AsyncClient(
            verify=client.verify_ssl, cookies=self._client.get_cookies()
        )

    def _get_httpx_kwargs(
        self,
        method: str,
        path: str,
        path_params: t.Optional[t.Dict[str, str]],
        query_params: t.Optional[t.Dict[str, str]],
        header_params: t.Optional[t.Dict[str, str]],
        json_body: t.Optional[str],
    ) -> t.Dict[str, t.Any]:
        if path_params is not None:
            path = path.format(**path_params)
        url = f"{self._client.base_url}{path}"

        headers: t.Dict[str, str] = {
            **self._client.get_headers(),
            "svix-req-id": f"{random.getrandbits(64)}",
        }
        if header_params is not None:
            headers.update(header_params)

        httpx_kwargs = {
            "method": method.upper(),
            "url": url,
            "headers": headers,
            "timeout": self._client.get_timeout(),
            "follow_redirects": self._client.follow_redirects,
        }

        if query_params is not None:
            httpx_kwargs["params"] = query_params

        if json_body is not None:
            encoded_body = json_body.encode("utf-8")
            httpx_kwargs["content"] = encoded_body
            headers["content-type"] = "application/json"
            headers["content-length"] = str(len(encoded_body))

        return httpx_kwargs

    async def _request_asyncio(
        self,
        method: str,
        path: str,
        path_params: t.Optional[t.Dict[str, str]] = None,
        query_params: t.Optional[t.Dict[str, str]] = None,
        header_params: t.Optional[t.Dict[str, str]] = None,
        json_body: t.Optional[str] = None,
    ) -> httpx.Response:
        httpx_kwargs = self._get_httpx_kwargs(
            method,
            path,
            path_params=path_params,
            query_params=query_params,
            header_params=header_params,
            json_body=json_body,
        )

        response = await self._httpx_async_client.request(**httpx_kwargs)

        for retry_count, sleep_time in enumerate(self._client.retry_schedule):
            if response.status_code < 500:
                break

            await asyncio.sleep(sleep_time)
            httpx_kwargs["headers"]["svix-retry-count"] = str(retry_count)
            response = await self._httpx_async_client.request(**httpx_kwargs)

        return _filter_response_for_errors_response(response)

    def _request_sync(
        self,
        method: str,
        path: str,
        path_params: t.Optional[t.Dict[str, str]] = None,
        query_params: t.Optional[t.Dict[str, str]] = None,
        header_params: t.Optional[t.Dict[str, str]] = None,
        json_body: t.Optional[str] = None,
    ) -> httpx.Response:
        httpx_kwargs = self._get_httpx_kwargs(
            method,
            path,
            path_params=path_params,
            query_params=query_params,
            header_params=header_params,
            json_body=json_body,
        )

        response = self._httpx_client.request(**httpx_kwargs)
        for retry_count, sleep_time in enumerate(self._client.retry_schedule):
            if response.status_code < 500:
                break

            time.sleep(sleep_time)
            httpx_kwargs["headers"]["svix-retry-count"] = str(retry_count)
            response = self._httpx_client.request(**httpx_kwargs)

        return _filter_response_for_errors_response(response)


def _filter_response_for_errors_response(response: httpx.Response) -> httpx.Response:
    if 200 <= response.status_code <= 299:
        return response
    else:
        if response.status_code == 422:
            raise HTTPValidationError.init_exception(
                response.json(), response.status_code
            )
        else:
            raise HttpError.init_exception(response.json(), response.status_code)



# this file is @generated
import typing as t

from .application_out import ApplicationOut
from .common import BaseModel


class ListResponseApplicationOut(BaseModel):
    data: t.List[ApplicationOut]

    done: bool

    iterator: t.Optional[str]

    prev_iterator: t.Optional[str] = None



# this file is @generated
import typing as t
from datetime import datetime

from .common import BaseModel


class AppUsageStatsIn(BaseModel):
    app_ids: t.Optional[t.List[str]] = None
    """Specific app IDs or UIDs to aggregate stats for.

    Note that if none of the given IDs or UIDs are resolved, a 422 response will be given."""

    since: datetime

    until: datetime



# this file is @generated
import typing as t

from .common import BaseModel


class IntegrationUpdate(BaseModel):
    feature_flags: t.Optional[t.List[str]] = None
    """The set of feature flags the integration will have access to."""

    name: str
